#
# spec file for package ldc
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define so_ver        83
%define lname_runtime libdruntime-%{name}
%define lname_phobos  libphobos2-%{name}
%define _bashcompletionsdir %{_datadir}/bash-completion/completions

# Do bootstrap (even in Tumbleweed, and Leap 15+), otherwise LDC will build
# against old installed .so instead of new built one
%bcond_without ldc_bootstrap

%ifarch %{ix86} %arm
# 32-bit needs 1.12.0 intermediate build due to: https://github.com/ldc-developers/ldc/issues/2947
%bcond_without 1_12_0_intermediate
%else
%bcond_with 1_12_0_intermediate
%endif

%bcond_with ldc_tests

Name:           ldc
Version:        1.13.0
Release:        lp151.3.24
Summary:        The LLVM D Compiler
License:        BSD-3-Clause AND Artistic-1.0
Group:          Development/Languages/Other
Url:            https://wiki.dlang.org/LDC
Source0:        https://github.com/ldc-developers/ldc/releases/download/v%{version}/ldc-%{version}-src.tar.gz
Source1:        %{name}-rpmlintrc
Patch0:         ldc-1.9.0-fix_arm_build.patch
BuildRequires:  cmake
BuildRequires:  help2man
BuildRequires:  libconfig++-devel
BuildRequires:  libcurl-devel
BuildRequires:  libstdc++-devel
BuildRequires:  llvm-clang
BuildRequires:  llvm-devel >= 3.9
BuildRequires:  ncurses-devel
BuildRequires:  sqlite3-devel
BuildRequires:  zlib-devel
BuildRequires:  pkgconfig(bash-completion)
# Should be installed, at least runtime
Recommends:     ldc-phobos-devel = %{version}
Recommends:     ldc-runtime-devel = %{version}
Recommends:     %{name}-bash-completion
# Since version 1.13.0, ldc uses ld.gold by default
Requires:       binutils-gold
%if %{with ldc_bootstrap}
# v0.17.6 is the last version buildable with a C++ compiler, so use it for bootstrapping
Source10:       https://github.com/ldc-developers/ldc/releases/download/v0.17.6/ldc-0.17.6-src.tar.gz
%if %{with 1_12_0_intermediate}
# 1.12.0 is needed to build on 32-bit: https://github.com/ldc-developers/ldc/issues/2947
Source11:       https://github.com/ldc-developers/ldc/releases/download/v1.12.0/ldc-1.12.0-src.tar.gz
%endif
%endif
%if %{with ldc_tests}
BuildRequires:  gcc-c++
BuildRequires:  gdb
%endif
%if %{without ldc_bootstrap}
BuildRequires:  binutils-gold
BuildRequires:  ldc
BuildRequires:  ldc-phobos-devel
BuildRequires:  ldc-runtime-devel
%endif
%if %{with ldc_tests}
BuildRequires:  python
%endif
%if %{with ldc_tests}
BuildRequires:  timezone
BuildRequires:  unzip
%endif
# ppc64 is disabled due to boo#1113531
ExclusiveArch:  %{ix86} x86_64 %arm aarch64

%description
LDC is an LLVM based compiler for the D programming language. It uses the
frontend of the reference compiler (DMD), thereby supporting the same language
features, but profits from LLVM's superior optimizing and code generation
capabilities.

%package -n %{lname_runtime}%{so_ver}
Summary:        Minimal D runtime library
Group:          System/Libraries

%description -n %{lname_runtime}%{so_ver}
The minimal runtime library required to support the D programming language.

%package runtime-devel
Summary:        Development files for the D runtime library
Group:          Development/Libraries/Other
Requires:       %{lname_runtime}%{so_ver} = %{version}
Recommends:     ldc-phobos-devel = %{version}

%description runtime-devel
This package contains the druntime development files necessary for developing
with LDC.

%package -n %{lname_phobos}%{so_ver}
Summary:        The D standard library
Group:          System/Libraries

%description -n %{lname_phobos}%{so_ver}
This package includes ldc's phobos library - The D standard library.

%package phobos-devel
Summary:        Development files for the D standard library
Group:          Development/Libraries/Other
Requires:       %{lname_phobos}%{so_ver} = %{version}
Requires:       %{name}-runtime-devel = %{version}

%description phobos-devel
This package contains the Phobos development files necessary for developing
with LDC.

%package bash-completion
Summary:        LDC Bash completion
Group:          System/Shells
Requires:       bash-completion

%description bash-completion
Optional dependency offering bash completion for ldc2


%prep
%setup -q -n ldc-%{version}-src
%patch0 -p1
%if %{with ldc_bootstrap}
tar xf %{SOURCE10}
pushd ldc-0.17.6-src
popd
%if %{with 1_12_0_intermediate}
tar xf %{SOURCE11}
pushd ldc-1.12.0-src
%patch0 -p1
popd
%endif
%endif

%build
%if %{with ldc_bootstrap}
pushd ldc-0.17.6-src
#Needs to be compiled with clang, but opensuse_rules.cmake forces gcc so disable rule
touch ./no-suse-rules
mkdir build && pushd build
# FIXME: you should use %%cmake macros
cmake \
    -DCMAKE_USER_MAKE_RULES_OVERRIDE=./no-suse-rules \
    -DCMAKE_C_COMPILER="%{_bindir}/clang" \
    -DCMAKE_CXX_COMPILER="%{_bindir}/clang++" \
    -DINCLUDE_INSTALL_DIR:PATH=%{_includedir}/d \
    -DCMAKE_CXX_FLAGS="-std=c++11" \
    -DCMAKE_C_FLAGS="-fPIC" \
    ..
make %{?_smp_mflags}
popd
popd
%if %{with 1_12_0_intermediate}
pushd ldc-1.12.0-src
#Needs to be compiled with clang, but opensuse_rules.cmake forces gcc so disable rule
touch ./no-suse-rules
mkdir build && pushd build
# FIXME: you should use %%cmake macros
cmake \
    -DCMAKE_USER_MAKE_RULES_OVERRIDE=./no-suse-rules \
    -DCMAKE_C_COMPILER="%{_bindir}/clang" \
    -DCMAKE_CXX_COMPILER="%{_bindir}/clang++" \
    -DINCLUDE_INSTALL_DIR:PATH=%{_includedir}/d \
    -DD_COMPILER:PATH=`pwd`/../../ldc-0.17.6-src/build/bin/ldmd2 \
    -DCMAKE_CXX_FLAGS="-std=c++11" \
    -DCMAKE_C_FLAGS="-fPIC" \
    ..
make %{?_smp_mflags}
popd
popd
%endif
%endif
#Needs to be compiled with clang, but opensuse_rules.cmake forces gcc so disable rule
touch no-suse-rules
%cmake \
    -DCMAKE_USER_MAKE_RULES_OVERRIDE=./no-suse-rules \
    -DCMAKE_C_COMPILER="%{_bindir}/clang" \
    -DCMAKE_CXX_COMPILER="%{_bindir}/clang++" \
    -DINCLUDE_INSTALL_DIR:PATH=%{_includedir}/d \
%if %{with ldc_bootstrap}
%if %{with 1_12_0_intermediate}
    -DD_COMPILER:PATH=`pwd`/../ldc-1.12.0-src/build/bin/ldmd2 \
%else
    -DD_COMPILER:PATH=`pwd`/../ldc-0.17.6-src/build/bin/ldmd2 \
%endif
%endif
    -DCMAKE_CXX_FLAGS="-std=c++11"
make %{?_smp_mflags}

%if %{with ldc_tests}
%check
pushd build/
make %{?_smp_mflags} test
popd
%endif

%install
%cmake_install
# Install bash completion in the right folder
install -d %{buildroot}%{_bashcompletionsdir}
mv %{buildroot}/etc/bash_completion.d/ldc2 %{buildroot}%{_bashcompletionsdir}
rmdir %{buildroot}/etc/bash_completion.d/
# Build man pages
help2man %{buildroot}%{_bindir}/ldc2  > ldc2.1  && gzip ldc2.1
help2man %{buildroot}%{_bindir}/ldmd2 > ldmd2.1 && gzip ldmd2.1
install -d %{buildroot}%{_mandir}/man1
install -m 644 -t %{buildroot}%{_mandir}/man1/ ldmd2.1.gz ldc2.1.gz
rm -rf %{buildroot}%{_libexecdir}/debug

%post -n %{lname_runtime}%{so_ver} -p /sbin/ldconfig
%post -n %{lname_phobos}%{so_ver}  -p /sbin/ldconfig
%postun -n %{lname_runtime}%{so_ver} -p /sbin/ldconfig
%postun -n %{lname_phobos}%{so_ver}  -p /sbin/ldconfig

%files
%license LICENSE
%doc README.md
%{_mandir}/man1/*.1%{?ext_man}
%config %{_sysconfdir}/ldc2.conf
%{_bindir}/ldc*
%{_bindir}/ldmd2

%files -n %{lname_runtime}%{so_ver}
%{_libdir}/%{lname_runtime}-shared.so.*
%{_libdir}/%{lname_runtime}-debug-shared.so.*

%files runtime-devel
%{_libdir}/%{lname_runtime}-shared.so
%{_libdir}/%{lname_runtime}-debug-shared.so
%dir %{_includedir}/d
%{_includedir}/d/core
%{_includedir}/d/ldc
%{_includedir}/d/object.d

%files -n %{lname_phobos}%{so_ver}
%{_libdir}/%{lname_phobos}-shared.so.*
%{_libdir}/%{lname_phobos}-debug-shared.so.*

%files phobos-devel
%{_libdir}/%{lname_phobos}-shared.so
%{_libdir}/%{lname_phobos}-debug-shared.so
%{_includedir}/d/etc
%{_includedir}/d/std

%files bash-completion
%dir %{_datadir}/bash-completion
%dir %{_datadir}/bash-completion/completions
%{_bashcompletionsdir}/ldc2

%changelog
* Fri Jan 25 2019 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Requires binutils.gold as ldc uses ld.gold by default since version 1.13.0
* Mon Jan 21 2019 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable bootstrap again for all flavours, otherwise ldc will require old
  .so file instead of new one
- Use ldc 1.12.0 as intermediate build on 32-bit, due to upstream bug:
  * https://github.com/ldc-developers/ldc/issues/2947
* Thu Jan 10 2019 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Remove wrong requires to 'ldc-utils' as there is no such package
* Tue Dec 18 2018 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Disable bootstrap in Tumbleweed and Leap 15+ as it is bootstrapped already
  and to workaround https://github.com/ldc-developers/ldc/issues/2947
* Tue Dec 18 2018 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add bash-completion sub-package
* Tue Dec 18 2018 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.13.0:
  * Big news
  - Frontend, druntime and Phobos are at version 2.083.1.
    (#2878, #2893, #2920, #2933) (new)
  - The Windows packages are now fully self-sufficient, i.e., a
    Visual Studio/C++ Build Tools installation isn't required anymore,
    as we now ship with MinGW-w64-based libraries, similar to DMD. Check out
    the included README.txt for all relevant details.
    (dlang/installer#346, #2886, Wiki: Cross-compiling with LDC)
  - Debug info improvements:
  - For GDB: printing global and imported symbols, non-member
    and member function calls. (#2826)
  - For Visual Studio and mago: names, by-value params, nested
    variables. (#2895, #2908, #2909, #2912)
  - Associative arrays now showing up properly (at least with mago),
    not as opaque void* anymore. (#2869)
  - -gc now translates D names to C++ ones, e.g., to use the regular
    Visual Studio debugger (bypassing mago) and as preparation for VS Code
    debugging with Microsoft's C/C++ plug-in (screenshots). Thanks to
    Oleksandr for this contribution and the AA fix! (#2869)
  - New command-line option -fvisibility=hidden to hide functions/globals
    not marked as export (for non-Windows targets), primarily to reduce the
    size of shared libraries. Thanks to Andrey for stepping up! (#2894, #2923)
  - Dropped support for LLVM 3.7 and 3.8. (#2872)
  - LLVM for prebuilt packages upgraded to v7.0.1. (new)
  - Linux: now defaulting to ld.gold linker in general, not just with
  - flto=thin, as buggy older ld.bfd versions may wrongly strip out required
    symbols (change with -linker). (#2870)
  - Improved support for Android/x86[_64], musl libc and FreeBSD/AArch64.
    (#2917, ldc-developers/druntime#146) (new)
  - LDC-specific druntime: ldc.simd.inlineIR moved/renamed to ldc.llvmasm.__ir
    (with deprecated legacy alias). (#2931) (new)
  - New CMake option COMPILE_D_MODULES_SEPARATELY builds D files in the DDMD
    frontend separately to reduce the time required to build LDC with many
    CPU cores and/or for iterative development. (#2914)
  * Platform support
  - Supports LLVM 3.9 - 7.0.
  - Alpine linux/x64: built against Musl libc to support Docker images based on
    the Alpine distro, requires the llvm5-libs, musl-dev, binutils-gold and gcc
    packages to build and link D apps and the tzdata and curl-dev packages for
    certain stdlib modules.
  * Bug fixes
  - 32-bit Android/ARM regression introduced in v1.12. (#2892)
  - Non-Windows x86_64 ABI fixes wrt. what's passed in registers,
    relevant for C[++] interop. (#2864)
  - Some issues with beta1's newly bundled Windows libraries. (#2900, #2903)
  - Alignment of scope allocated class instances. (#2919)
  - Beta2's -fvisibility=hidden causing lots of linker errors.
    (#2922, #2923) (new)
- Update so_version to 83
- Disable ppc64 builds (boo#1113531)
* Wed Oct 17 2018 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.12.0:
  * Big news
  - Frontend, druntime and Phobos are at version 2.082.1.
    (#2818, #2837, #2858, #2873) (new)
    Significant performance improvements for some transcendental
    std.math functions in single and double precision, at least
    for x86. (dlang/phobos#6272 (comment))
  - Support for LLVM 7, which is used for the prebuilt packages.
    Due to an LLVM 7.0.0 regression, the prebuilt x86[_64] LDC
    binaries require a CPU with SSSE3, and so will your optimized
    binaries (unless compiling with -mattr=-ssse3). (#2850)
  - JIT compilation: new ldc.dynamic_compile.bind function with
    interface similar to C++ std::bind, allowing to generate
    efficient specialized versions of functions (much like
    Easy::jit for C++). (#2726)
  - LTO now working for Win64 too; the prebuilt package includes
    the required external LLD linker and the optional LTO default
    libs. Enable as usual with -flto=<thin|full>
    [-defaultlib=druntime-ldc-lto,phobos2-ldc-lto]. (#2774)
  - Config file: new lib-dirs array for directories to be searched
    for libraries, incl. LLVM compiler-rt libraries. (#2790)
  * Platform support
  - Supports LLVM 3.7 - 7.0.
  - Windows: Supports Visual Studio/C++ Build Tools 2015 and 2017.
  - Alpine linux/x64: built against Musl libc to support Docker
    images based on the Alpine distro, requires the llvm5-libs,
    musl-dev, and gcc packages to build and link D apps and the
    tzdata and libcurl packages for certain stdlib modules.
  - Android/ARM: This release slightly changes the way emulated TLS
    is interfaced, but is missing a patch for 32-bit ARM. See the
    wiki for instructions on patching that file manually before
    cross-compiling the runtime libraries for 32-bit Android/ARM.
  * Bug fixes
  - Fix IR-based PGO on Windows (requires our LLVM fork). (#2539)
  - Fix C++ class construction with D new on Posix. (#2801)
  - Android: No more text relocations in Phobos zlib, required for
    API level 23+. (#2822, #2835)
  - Declare extern const/immutable globals as IR constants.
    (#2849, #2852)
  - Fix issue when emitting both object and textual assembly files
    at once (-output-o -output-s). (#2847)
  - Support address of struct member as key/value in AA literal.
    (#2859, #2860)
  - Fix ICE when computing addresses relative to functions/labels.
    (#2865, #2867) (new)
  - beta2 regression wrt. cross-compiling to Android. (#2863) (new)
- Fix so_version to 82
* Mon Aug 27 2018 guillaume.gardet@opensuse.org
- Fix so_version to 81
* Fri Aug 24 2018 guillaume@opensuse.org
- Update to 1.11.0:
  * Big news:
  - Frontend, druntime and Phobos are at version 2.081.2.
    (#2752, #2772, #2776, #2791, #2815) (new)
  - Add some support for classes without TypeInfos, for
  - betterC and/or a minimal (d)runtime. (#2765)
  - LLVM for prebuilt packages upgraded to v6.0.1.
    The x86_64 packages feature some more LLVM targets for
    cross-compilation (experiments): MIPS, MSP430, RISC-V
    and WebAssembly. (#2760)
  - Rudimentary support for compiling & linking directly to
    WebAssembly. See the dedicated Wiki page for how to get
    started. (#2766, #2779, #2785)
  - AArch64 (64-bit ARM) now mostly working on Linux/glibc
    and Android. Current ltsmaster is able to bootstrap v1.11,
    which can also bootstrap itself; most tests pass.
    (Preliminary) CI has been set up. (#2802, #2817, #2813) (new)
  - LDC on Windows now uses 80-bit compile-time reals. This allows
    for seamless cross-compilation to other x86(_64) targets,
    e.g., without real.min underflowing to 0 and real.max
    overflowing to infinity. (#2752)
  - New @naked UDA in ldc.attributes & enhanced functionality
    for @llvmAttr("<name>"). (#2773)
  * Platform support:
  - Supports LLVM 3.7 - 6.0.
  - Windows: Supports Visual Studio/C++ Build Tools 2015 and 2017.
  - Alpine linux/x64: built against Musl libc to support Docker
    images based on the Alpine distro, requires the llvm5-libs,
    musl-dev, and gcc packages to build and link D apps and the
    tzdata and libcurl packages for certain stdlib modules.
  * Bug fixes:
  - extern(C++) on Posix: Pass non-PODs indirectly by value.
    (#2728)
  - extern(C++) on Windows/MSVC: Methods return all structs via
    hidden sret pointer. (#2720, #1935)
  - Make GC2Stack IR optimization pass work as intended. (#2750)
  - Work around inline assembly regression with LLVM 6 on Win32.
    The prebuilt Win32 package is now using LLVM 6.0.1 too. (#2629, #2770)
  - Fix overzealous check for multiple main() functions. (#2778) (new)
  - Fix corrupt prefix in integrated LLD's console output. (#2781) (new)
  - No context ptr for nested non-extern(D) functions. (#2808, #2809) (new)
  * Known issues:
  - LDC does not zero the padding area of a real variable. This may
    lead to wrong results if the padding area is also considered.
    See #770. Does not apply to real members inside structs etc.
- Update bootstrap version to 0.17.6:
  * Added support for LLVM 6.0 and 7.0. (#2600, #2825)
  * Backported AArch64 fixes from master; most tests passing on
    Linux/glibc and Android
  * Fix generation of debug info. (#2594)
  * Added support for bootstrapping on DragonFly BSD
  * Fixed missing definition in std.datetime on Solaris
  * Fixed std.datetime unittest failure. (ldc-developers/phobos#59)
  * Fixed tests for PowerPC. (#2634, #2635)
  * Improvements for MIPS.
  * Make core.stdc.stdarg.va_* functions nothrow to enable
    compiling the 2.082 frontend. (#2821)
  * CI updates.
- Remove upstreamed patches:
  * ldc-0.17.5-add_missing_include.patch
  * ldc-0.17.5-add_support_to_LLVM6.patch
  * ldc-0.17.5-backport_pr_2315.patch
  * ldc-0.17.5-default-to-pic.patch
- Enable aarch64 since it can be boostrapped with ldc 0.17.6
- Fix x86_64 build by adding '-fPIC' to C_FLAGS of bootstrap
* Mon Jul 30 2018 matthias.eliasson@gmail.com
- Remove _sysconfdir from phobos-devel filelist and add etc since its a
  d package name. spec-cleaner was to eager to clean.
* Sat Jul 28 2018 matthias.eliasson@gmail.com
- Update so_ver to 80 for libdruntime-ldc and libphobos2-ldc to
  match version in 1.10.0 release of ldc.
- Run spec-cleaner
* Tue Jul  3 2018 guillaume.gardet@opensuse.org
- Update to 1.10.0:
  * Big news:
  - Frontend, druntime and Phobos are at version 2.080.1.
    (#2665, #2719, #2737) (new)
  - No support for Objective-C class/static methods yet. (#2670)
  - Breaking Win64 extern(D) ABI change: Pass vectors directly
    in registers, analogous to the MS vector calling convention.
    (#2714)
  - Config file: For cross-compilation, support additional
    sections named as regex for specific target triples, e.g.,
    "86(_64)?-.*-linux": { … };; see the comment in etc/ldc2.conf.
    (#2718)
  * Platform support:
  - Supports LLVM 3.7 - 6.0.
  - Windows: Supports Visual Studio/C++ Build Tools 2015 and 2017.
  - Alpine linux/x64: built against Musl libc to support Docker
    images based on the Alpine distro, requires the llvm5-libs,
    musl-dev, and gcc packages to run and link D apps and the
    tzdata and libcurl packages for certain stdlib modules.
  * Bug fixes
* Wed Jun  6 2018 guillaume.gardet@opensuse.org
- Add tests, disabled by default (since we are missing python2-lit)
* Wed Jun  6 2018 guillaume.gardet@opensuse.org
- Add patch to fix ARM builds:
  * ldc-1.9.0-fix_arm_build.patch
* Tue Jun  5 2018 guillaume.gardet@opensuse.org
- Update to 1.9.0 and use 0.17.5 to bootstrap
* Fri Jun  1 2018 guillaume.gardet@opensuse.org
- Add patch to fix rpmlint error:
  * ldc-0.17.5-add_missing_include.patch
- Add patches to add LLVM6 support:
  * ldc-0.17.5-add_support_to_LLVM6.patch
  * ldc-0.17.5-backport_pr_2315.patch
- Add patch to be able to use ldc:
  * ldc-0.17.5-default-to-pic.patch
* Thu Mar 22 2018 guillaume.gardet@opensuse.org
- update to version 0.17.5:
  * Added LLVM 5.0 support.
  * druntime: fixes for Android and addition of core.math.yl2x[p1]() for x86(_64) targets.
  * dmd-testsuite: backported runnable/cppa.d fix for GCC > 5.
  * CI updates.
* Fri Jun  9 2017 jengelh@inai.de
- Rectify RPM groups
* Mon May 29 2017 mailaender@opensuse.org
- update to version 0.17.4
* Fri May 26 2017 mailaender@opensuse.org
- update to version 0.17.3
- https://en.opensuse.org/SourceUrls
- drop workaround-buggy-llvm-config.patch
* Mon Feb 15 2016 schuetzm@gmx.net
- update to 0.17.0
- based on DMD 2.068.2
- more information here:
  https://github.com/ldc-developers/ldc/releases/tag/v0.17.0
* Sun Dec  6 2015 rpm@fthiessen.de
- Build runtime and phobos libraries as shared libraries, therefore
  refactored spec file:
  * Basic cleanup
  * Split lib* packages, see openSUSE shared library policy
  * Added ldc-rpmlintrc for false positiv.
* Wed Oct 28 2015 schuetzm@gmx.net
- update to 0.16.1
- this is a bugfix only release
* Sat Oct 24 2015 schuetzm@gmx.net
- update 0.16.0
- based on DMD 2.067.1
* Sun Feb 22 2015 schuetzm@gmx.net
- initial submission based on Duncan Mac-Vicar's work
